package Nimbus::PDS;

use vars qw($VERSION @ISA @EXPORT);
use Nimbus::API;

require Exporter;
@ISA = qw(Exporter);

use constant {
    PDS_PPCH => 8,
	PDS_PPI => 3
};

#############################################################
#Export selected functions from Nimbus::API;
#############################################################
@EXPORT = qw (
	nimLog 
	nimLogSet 
	nimLogSetLevel
	PDS_PPCH
	PDS_PPI
	PDS_INT
	PDS_F
	PDS_PCH
	PDS_PDS
);

$VERSION ='2.02';

#############################################################
#
#############################################################
sub new {
	my $class = shift;
	my $pds   = shift; 
	my $cleanup = shift || 0;
	my $self  = { };
	my $localPds = 0;

	bless($self,$class);

	if (!defined($pds)) {
		$pds = pdsCreate();
		$localPds = 1;
	}

	$self->{pds}  = $pds;
	$self->{getTableIndex}  = 0;
	$self->{localPds} = $localPds + $cleanup;

	return $self;
}

#############################################################
#
#############################################################
sub DESTROY {
	my $self = shift;
	nimLog (2,"$self: Object is destroyed");
	if ($self->{localPds}) {
		nimLog(2,"$self: Local PDS being removed");
		pdsDelete($self->{pds});
	}
}

#############################################################
#
#############################################################
sub post {
	my $self    = shift;
	my $subject = shift;

	if (!defined($subject)) {
		return 1;
	}

	nimLog(2,"PDS::post -  subject: $subject");

	return nimPostMessage($subject,$self->{pds});
}

#############################################################
#
#############################################################
sub dump {
	my $self  = shift;
	nimLog(2,"PDS::dump");

	pdsDump($self->{pds});	
}

#############################################################
#
#############################################################
sub data {
	my $self  = shift;

	return $self->{pds};	
}

#############################################################
#
#############################################################
sub remove {
	my $self  = shift;
	my $name  = shift;

	return pdsRemove($self->{pds},$name);
}

#############################################################
#
#############################################################
sub rewind {
	my $self  = shift;

	return pdsRewind($self->{pds});
}

#############################################################
#
#############################################################
sub reset {
	my $self  = shift;

	return pdsReset($self->{pds});
}

#############################################################
#
#############################################################
sub get {
	my $self  = shift;
	my $name  = shift;
	my $type  = shift || PDS_PCH;

	if (!defined($name)) {
		return;
	}

	if ($type == PDS_PCH) {
		$value = pdsGet_PCH($self->{pds},$name);
		$type  = "PCH";
	}elsif ($type == PDS_INT) {
		$value = pdsGet_INT($self->{pds},$name);
		$type  = "INT";
	}elsif ($type == PDS_F) {
		$value = pdsGet_F($self->{pds},$name);
		$type  = "FLOAT";
	}elsif ($type == PDS_PDS) {
		my $rd = pdsGet_PDS($self->{pds},$name);
		if ($rd) {
			nimLog(2,"PDS::get -  PDS,  $name");
			return Nimbus::PDS->new($rd,1);
		}

	}

	nimLog(2,"PDS::get -  $type,  $name = $value");

	return $value;
}

sub getString {
	my $self  = shift;
	my $name  = shift;
	return get($self,$name,PDS_PCH);
}

sub getNumber {
	my $self  = shift;
	my $name  = shift;
	return get($self,$name,PDS_INT);
}

sub getFloat {
	my $self  = shift;
	my $name  = shift;
	return get($self,$name,PDS_F);
}

sub getPDS {
	my $self  = shift;
	my $name  = shift;
	return get($self,$name,PDS_PDS);
}

#############################################################
#
#############################################################
sub getTable {
	my $self  = shift;
	my $name  = shift;
	my $type  = shift || PDS_PCH;
	my $indx  = shift;

	if (!defined($name)) {
		return;
	}

	if (!defined($indx)) {
		$indx =	$self->{getTableIndex};
	}

	##############################################
	# Retrieve type from pds by using name...

	my ($rc,$rd) = pdsGetTable($self->{pds},$type,$name,$indx);

	if ($rc==0) {
		$self->{getTableIndex}++;
		if ($type == PDS_PDS) {
			return Nimbus::PDS->new($rd,1);
		}
		return $rd;
	}else {
		$self->{getTableIndex} = 0;
	}
	return;
}

#############################################################
#
#############################################################
sub put {
	my $self  = shift;
	my $name  = shift;
	my $value = shift;
	my $type  = shift;

	if (!defined($name) || !defined($value)) {
		return 0;
	}

	if (!defined($type)) {
		if (ref($value) eq "Nimbus::PDS") {
			pdsPut_PDS($self->{pds},$name,$value->{pds});
			nimLog(2,"PDS::put - name: $name, value: $value");
			return 1;
		}elsif (ref($value) eq "SCALAR") {
			$type = PDS_PCH;	
		}
	}

	if ($type == PDS_PCH) {
		pdsPut_PCH($self->{pds},$name,$value);
	}elsif ($type == PDS_INT) {
		pdsPut_INT($self->{pds},$name,$value);
	}elsif ($type == PDS_F) {
		pdsPut_F($self->{pds},$name,$value);
	}elsif ($type == PDS_PDS) {
		if (ref($value) eq "Nimbus::PDS") {
			pdsPut_PDS($self->{pds},$name,$value->{pds});
		}else {
			pdsPut_PDS($self->{pds},$name,$value);
		}
	}else {
		nimLog(2,"PDS::put - unknown type");
		return 0;
	}

	nimLog(2,"PDS::put - name: $name, value: $value, $type: $type");
	return 1;
}

#############################################################
#
#############################################################
sub number {
	return putNumber(@_);
}

#############################################################
#
#############################################################
sub string {
	return putString(@_);
}

#############################################################
#
#############################################################
sub float {
	return putFloat(@_);
}

#############################################################
#
#############################################################
sub putString {
	my $self  = shift;
	my $name  = shift;
	my $value = shift||"";
	
	if (!(defined($name) && defined($value))) {
		return;
	}
	return $self->put($name,$value,PDS_PCH);
}

#############################################################
#
#############################################################
sub putNumber {
	my $self  = shift;
	my $name  = shift;
	my $value = shift || 0;
	
	if (!(defined($name) && defined($value))) {
		return;
	}
	return $self->put($name,$value,PDS_INT);
}

#############################################################
#
#############################################################
sub putFloat {
	my $self  = shift;
	my $name  = shift;
	my $value = shift || 0.0;
	
	if (!(defined($name) && defined($value))) {
		return;
	}
	return $self->put($name,$value,PDS_F);
}

#############################################################
#
#############################################################
sub putPDS {
	my $self  = shift;
	my $name  = shift;
	my $value = shift;
	
	if (!(defined($name) && defined($value))) {
		return;
	}
	return $self->put($name,$value,PDS_PDS);
}

#############################################################
#
#############################################################
sub putTable {
	my $self  = shift;
	my $name  = shift;
	my $value = shift;
	my $type  = shift || PDS_PCH;

	if (ref($value) eq "Nimbus::PDS")  {
		return pdsPutTable($self->{pds},PDS_PDS,$name,$value->{pds});
	}

	return pdsPutTable($self->{pds},$type,$name,$value);
}

############################################################
# Return PDS elements in hash.
############################################################
sub asHash {
	my $self = shift;
	my $hptr = shift || {};
	my $pds  = shift || $self->{pds};
	my $lev  = shift || 1;

	my ($rc, $key, $type, $s, $value);
	my $line = "-" x $lev;
	while($rc == 0) {
		($rc, $key, $type, $size, $value) = pdsGetNext($pds);
		next if $rc != PDS_ERR_NONE;
		# print "PDS Type => $type, key => $key, value => $value, size => $size\n";
		if ($type == PDS_PDS) {
			if (!defined($hptr->{$key})) {
				nimLog(3,"PDS::asHash $line>Adding PDS: $key\n");
				$hptr->{$key} = {};
			}
			asHash($self, $hptr->{$key}, $value, $lev + 1);
			pdsDelete($value);
		}
		elsif ($type == PDS_PPCH || $type == PDS_PPI) {
			nimLog(3,"PDS::asHash $line>Adding Array: $key\n");
			my $tableIndex = 0;
			my @tableValues = ();
			my ($rc_table, $rd);
			WPDS_PCH: while($rc_table == 0) {
				($rc_table, $rd) = pdsGetTable($pds, PDS_PCH, $key, $tableIndex);
				last WPDS_PCH if $rc_table != PDS_ERR_NONE;
				push(@tableValues, $rd);
				$tableIndex++;
			};
			$hptr->{$key} = \@tableValues;
		}
		else {
			nimLog(3, "PDS::asHash $line>Adding key/value: $key = $value");
			$hptr->{$key} = $value;
		}
	};
	return $hptr;
}

1;

__END__

=head1 NAME

Nimbus::PDS - Object interface wrapping the PDS

=head1 SYNOPSIS

 use Nimbus::PDS

 my $pds = Nimbus::PDS->new( [$pdsData] );

          $pds->data();
          $pds->dump();
          $pds->rewind();
          $pds->reset();
          $pds->remove($name);
          $pds->string($name,$value);
          $pds->number($name,$value);
          $pds->float ($name,$value);
          $pds->put ($name,$value [,$type]);
          $pds->putTable ($name,$value [,$type]);
 $value = $pds->getTable ($name [,$type]);
 $value = $pds->get ($name [,$type]);
 $hptr  = $pds->asHash();

=head1 DESCRIPTION

The PDS object is a class wrapper around the Nimbus::API PDS functions.

=head1 CLASS METHODS

=head2 get

The get method....

=head2 put

The put method....

=head2 dump

The dump method....

=head2 putTable

The putTable method....

=head2 asHash

The asHash method will produce an associative array (hash) by traversing
the PDS. If the PDS contains other PDS's, then the hiearchy will be preserved
by nesting.

 Example:

 use Nimbus::PDS;
 my $pds = Nimbus::PDS->new();
 $pds->string("name","Donald Duck");
 $pds->number("age",60);

 my $h = $pds->asHash();
 print "name: $h->{name}, age: $h->{age}\n";

=head1 AUTHOR
 
 Nimbus Software AS.
 mailto:nimsoft@nimsoft.no
 http://www.nimsoft.no

=head1 SEE ALSO
	 
Nimbus::API, perl(1).

